(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = global || self, factory(global.howLongUntilLunch = {}));
}(this, (function (exports) { 'use strict';

    var VbAlert = {
        createIsHandler(Alert, el, binding) {
            //console.log('alert createIsHandler', el, binding)
            return {
                beforeMount() {
                    if (el.classList && !el.classList.contains('alert')) el.classList.add('alert');
                    if (!el.$vb) el.$vb = {};
                    let ins = Alert.getInstance(el);
                    if (!ins) ins = new Alert(el, binding.value);
                    el.$vb.alert = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('alert')) el.classList.add('alert');
                },
                beforeUnmount() {
                    let ins = Alert.getInstance(el);
                    if (ins) ins.dispose();
                }
            }
        },

        createDismissHandler(Alert, el, binding) {
            //console.log('alert createIsHandler', el, binding)
            let getParentAlert = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('alert')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dismiss', binding)
                if (binding.value) {
                    let allowedToClose = binding.value(e);
                    //console.log('allowedToClose', allowedToClose)
                    if (!allowedToClose) return
                }
                let alertEl = getParentAlert(e.target);
                if (alertEl) {
                    let ins = Alert.getInstance(alertEl);
                    if (ins) ins.close();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }
    };

    var VbCarousel = {

        createIsHandler(Carousel, el, binding) {
            //console.log('carousel createIsHandler')
            return {
                beforeMount() {
                    //console.log('carousel beforeMount', el)
                    if (el.classList && !el.classList.contains('carousel')) el.classList.add('carousel');
                    if (!el.$vb) el.$vb = {};
                    let ins = Carousel.getInstance(el);
                    if (!ins) ins = new Carousel(el, binding.value);
                    el.$vb.carousel = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('carousel')) el.classList.add('carousel');
                },
                beforeUnmount() {
                    let ins = Carousel.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.carousel = undefined;
                }
            }
        },

        createSlideHandler(Carousel, el, binding) {
            //console.log('carousel createSlideHandler', el, binding)
            let getParentCarousel = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('carousel')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('carousel', binding)
                let carouselEl = getParentCarousel(e.target);
                if (carouselEl) {
                    let ins = Carousel.getInstance(carouselEl);
                    if (ins && binding.arg === 'prev') ins.prev();
                    else if (ins && binding.arg === 'next') ins.next();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },

        createSlideToHandler(Carousel, el, binding) {
            //console.log('carousel createSlideHandler', el, binding)
            let getParentCarousel = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('carousel')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('carousel', binding)

                let carouselEl = getParentCarousel(e.target);
                if (carouselEl) {
                    let ins = Carousel.getInstance(carouselEl);
                    if (ins && binding.arg) {
                        let toIdx = Number(binding.arg);
                        if (!Number.isNaN(toIdx)) ins.to(toIdx);
                    }
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }


    };

    var VbCollapse = {

        createIsHandler(Collapse, el, binding) {
            //console.log('collapse createIsHandler')
            return {
                beforeMount() {
                    //console.log('collapse beforeMount', el)
                    if (el.classList && !el.classList.contains('collapse')) el.classList.add('collapse');
                    if (!el.$vb) el.$vb = {};
                    let ins = Collapse.getInstance(el);
                    if (!ins) ins = new Collapse(el, binding.value);
                    el.$vb.collapse = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('collapse')) el.classList.add('collapse');
                },
                beforeUnmount() {
                    let ins = Collapse.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.collapse = undefined;
                }
            }
        },

        createDismissHandler(Collapse, el, binding) {
            let getParentCollapse = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('collapse')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dismiss', binding)
                if (binding.value) {
                    let allowedToClose = binding.value(e);
                    //console.log('allowedToClose', allowedToClose)
                    if (!allowedToClose) return
                }
                let collapseEl = getParentCollapse(e.target);
                if (collapseEl) {
                    let ins = Collapse.getInstance(collapseEl);
                    if (ins) ins.hide();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }


    };

    var VbDropdown = {

        createIsHandler(Dropdown, el, binding) {
            //console.log('dropdown createIsHandler')
            return {
                beforeMount() {
                    //console.log('dropdown beforeMount', el)
                    if (el.classList && !el.classList.contains('dropdown')) el.classList.add('dropdown');
                    if (!el.$vb) el.$vb = {};
                    let ins = Dropdown.getInstance(el);
                    if (!ins) ins = new Dropdown(el, binding.value);
                    el.$vb.dropdown = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('dropdown')) el.classList.add('dropdown');
                    let ins = Dropdown.getInstance(el);
                    if (ins) ins.update();
                },
                beforeUnmount() {
                    let ins = Dropdown.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.dropdown = undefined;
                }
            }
        },

        createToggleHandler(Dropdown, el, binding) {
            //console.log('createToggleHandler', el, binding)
            let clickHandler = async function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dropdown toggle click', el, binding.value, el.dataset.bsTarget)
                let targetEl = null;
                if (binding.value) {
                    let refObj = binding.instance.$refs[binding.value];
                    targetEl = refObj && refObj.$el ? refObj.$el : refObj;
                } else if (el.dataset.bsTarget) {
                    targetEl = document.querySelector(el.dataset.bsTarget);
                }
                if (targetEl) {
                    let ins = Dropdown.getInstance(targetEl);
                    if (ins) ins.toggle(el);
                }
            };
            return {
                beforeMount() {
                    //console.log('dropdown toggle beforemount')
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },


    };

    var VbModal = {

        createIsHandler(Modal, el, binding, baseOptions) {
            //console.log('modal createIsHandler')

            // bootstrap by default doesn't allow layering of modals, fix this when modal is shown by adjusting the zindex
            // of each new modal and it's backdrop
            let backdropFix = function() {
                let modalsVisible = document.querySelectorAll('.modal');
                let zIndex = baseOptions.vbModalBaseZindex + (10 * modalsVisible.length);
                el.style.zIndex = String(zIndex);
                setTimeout(function() {
                    // ensure the backdrop is one back in the zindex
                    let bdEl = document.querySelector('.modal-backdrop:not(.vb-modal-stack)');
                    if (bdEl) {
                        bdEl.style.zIndex = String(zIndex - 1);
                        bdEl.classList.add('vb-modal-stack');
                    }
                    // remove this event handler
                    el.removeEventListener('show.bs.modal', backdropFix);
                }, 0);
            };
            let hiddenEventHandler = function() {
                let evt = document.createEvent('HTMLEvents');
                evt.initEvent('vb-hidden-bs-modal', true, true);
                el.dispatchEvent(evt);
            };
            let hideEventHandler = function(e) {
                let evt = document.createEvent('HTMLEvents');
                evt.initEvent('vb-hide-bs-modal', true, true);
                el.dispatchEvent(evt);
                if (evt.defaultPrevented) e.preventDefault();
            };
            let shownEventHandler = function() {
                let evt = document.createEvent('HTMLEvents');
                evt.initEvent('vb-shown-bs-modal', true, true);
                el.dispatchEvent(evt);
            };

            return {
                beforeMount() {
                    //console.log('modal beforeMount', el)
                    if (el.classList && !el.classList.contains('modal')) el.classList.add('modal');
                    if (!el.$vb) el.$vb = {};
                    let ins = Modal.getInstance(el);
                    if (!ins) ins = new Modal(el, binding.value);
                    el.$vb.modal = ins;
                    el.addEventListener('show.bs.modal', backdropFix);
                    el.addEventListener('shown.bs.modal', shownEventHandler);
                    el.addEventListener('hide.bs.modal', hideEventHandler);
                    el.addEventListener('hidden.bs.modal', hiddenEventHandler);
                },
                mounted() {
                    //console.log('modal mounted', el)
                    let modifiers = Object.keys(binding.modifiers).map(key => key);
                    if (modifiers && modifiers.length > 0 && modifiers[0] === 'show') {
                        let ins = Modal.getInstance(el);
                        if (ins) ins.show();
                    }
                },
                updated() {
                    //console.log('modal updated')
                    if (el.classList && !el.classList.contains('modal')) el.classList.add('modal');
                    let ins = Modal.getInstance(el);
                    if (ins) ins.handleUpdate();
                },
                beforeUnmount() {
                    //console.log('modal beforeUnmount', el)
                    // run in next loop to ensure that any hide.bs.modal events are already removed and can't call preventDefault
                    setTimeout(() => {
                        let ins = Modal.getInstance(el);
                        // we ideally would not call _isShown (private) however we need it closed with no transition
                        if (ins && ins._isShown) {
                            // Vue removes the element in the next loop, so we need to ensure that the animation doesn't
                            // run and Bootstrap can cleanup immediately
                            if (el.classList && el.classList.contains('fade')) el.classList.remove('fade');
                            ins.hide();
                        }
                        if (ins) ins.dispose();
                        el.$vb.modal = undefined;
                        //console.log('modal cleanup done', el)
                    });
                },
                unmounted() {
                    //console.log('modal unmounted', el)
                    el.removeEventListener('show.bs.modal', backdropFix);
                    el.removeEventListener('shown.bs.modal', shownEventHandler);
                    el.removeEventListener('hide.bs.modal', hideEventHandler);
                    el.removeEventListener('hidden.bs.modal', hiddenEventHandler);
                }

            }
        },

        createToggleHandler(Modal, el, binding) {
            //console.log('createToggleHandler', el, binding)
            let clickHandler = async function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('modal toggle click', el, binding.value, el.dataset.bsTarget)
                let targetEl = null;
                if (binding.value && typeof binding.value === 'string') {
                    let refObj = binding.instance.$refs[binding.value];
                    targetEl = refObj && refObj.$el ? refObj.$el : refObj;
                } else if (el.dataset.bsTarget) {
                    targetEl = document.querySelector(el.dataset.bsTarget);
                }
                if (targetEl) {
                    let ins = Modal.getInstance(targetEl);
                    if (!ins) ins = new Modal(targetEl);
                    if (ins) ins.toggle(el);
                }
            };
            return {
                beforeMount() {
                    //console.log('modal toggle beforemount')
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },

        createDismissHandler(Modal, el, binding) {
            let getParentModal = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('modal')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dismiss', binding)
                if (binding.value) {
                    let allowedToClose = binding.value(e);
                    //console.log('allowedToClose', allowedToClose)
                    if (!allowedToClose) return
                }
                let modalEl = getParentModal(e.target);
                if (modalEl) {
                    let ins = Modal.getInstance(modalEl);
                    if (ins) ins.hide();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }


    };

    var VbOffcanvas = {

        createIsHandler(Offcanvas, el, binding) {
            //console.log('offcanvas createIsHandler')
            return {
                beforeMount() {
                    //console.log('offcanvas beforeMount', el)
                    if (el.classList && !el.classList.contains('offcanvas')) el.classList.add('offcanvas');
                    if (!el.$vb) el.$vb = {};
                    let ins = Offcanvas.getInstance(el);
                    if (!ins) ins = new Offcanvas(el, binding.value);
                    el.$vb.offcanvas = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('offcanvas')) el.classList.add('offcanvas');
                },
                beforeUnmount() {
                    let ins = Offcanvas.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.offcanvas = undefined;
                }
            }
        },

        createToggleHandler(Offcanvas, el, binding) {
            //console.log('createToggleHandler', el, binding)
            let clickHandler = async function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('offcanvas toggle click', el, binding.value, el.dataset.bsTarget)
                let targetEl = null;
                if (binding.value) {
                    let refObj = binding.instance.$refs[binding.value];
                    targetEl = refObj && refObj.$el ? refObj.$el : refObj;
                } else if (el.dataset.bsTarget) {
                    targetEl = document.querySelector(el.dataset.bsTarget);
                }
                if (targetEl) {
                    let ins = Offcanvas.getInstance(targetEl);
                    if (ins) ins.toggle(el);
                }
            };
            return {
                beforeMount() {
                    //console.log('offcanvas toggle beforemount')
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },

        createDismissHandler(Offcanvas, el, binding) {
            let getParentOffcanvas = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('offcanvas')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dismiss', binding)
                if (binding.value) {
                    let allowedToClose = binding.value(e);
                    //console.log('allowedToClose', allowedToClose)
                    if (!allowedToClose) return
                }
                let offcanvasEl = getParentOffcanvas(e.target);
                if (offcanvasEl) {
                    let ins = Offcanvas.getInstance(offcanvasEl);
                    if (ins) ins.hide();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }


    };

    var VbPopover = {

        createIsHandler(Popover, el, binding) {
            //console.log('popover createIsHandler')
            return {
                beforeMount() {
                    //console.log('popover beforeMount', el)
                    if (el.classList && !el.classList.contains('popover')) el.classList.add('popover');
                    if (!el.$vb) el.$vb = {};
                    let ins = Popover.getInstance(el);
                    if (!ins) ins = new Popover(el, binding.value);
                    el.$vb.popover = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('popover')) el.classList.add('popover');
                    let ins = Popover.getInstance(el);
                    if (ins) ins.update();
                },
                beforeUnmount() {
                    let ins = Popover.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.popover = undefined;
                }
            }
        },

        createToggleHandler(Popover, el, binding) {
            //console.log('createToggleHandler', el, binding)
            let clickHandler = async function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('popover toggle click', el, binding.value, el.dataset.bsTarget)
                let targetEl = null;
                if (binding.value) {
                    let refObj = binding.instance.$refs[binding.value];
                    targetEl = refObj && refObj.$el ? refObj.$el : refObj;
                } else if (el.dataset.bsTarget) {
                    targetEl = document.querySelector(el.dataset.bsTarget);
                }
                if (targetEl) {
                    let ins = Popover.getInstance(targetEl);
                    if (ins) ins.toggle(el);
                }
            };
            return {
                beforeMount() {
                    //console.log('popover toggle beforemount')
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },


    };

    var VbScrollSpy = {

        createIsHandler(ScrollSpy, el, binding) {
            //console.log('scrollspy createIsHandler')
            return {
                beforeMount() {
                    //console.log('scrollspy beforeMount', el)
                    if (!el.$vb) el.$vb = {};
                    let ins = ScrollSpy.getInstance(el);
                    if (!ins) ins = new ScrollSpy(el, binding.value);
                    el.$vb.scrollspy = ins;
                },
                updated() {
                    let ins = ScrollSpy.getInstance(el);
                    if (ins) ins.refresh();
                },
                beforeUnmount() {
                    let ins = ScrollSpy.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.scrollspy = undefined;
                }
            }
        },

        createSpyHandler(ScrollSpy, el, binding) {
            //console.log('scrollspy createIsHandler')
            if (binding.arg === 'scroll') {
                return {
                    beforeMount() {
                        //console.log('scrollspy beforeMount', el)
                        if (!el.$vb) el.$vb = {};
                        let ins = ScrollSpy.getInstance(el);
                        if (!ins) ins = new ScrollSpy(el, binding.value);
                        el.$vb.scrollspy = ins;
                    },
                    updated() {
                        let ins = ScrollSpy.getInstance(el);
                        if (ins) ins.refresh();
                    },
                    beforeUnmount() {
                        let ins = ScrollSpy.getInstance(el);
                        if (ins) ins.dispose();
                        el.$vb.scrollspy = undefined;
                    }
                }
            } else {
                return null
            }
        },


    };

    var VbTab = {
        createIsHandler(Tab, el, binding) {
            //console.log('tab createIsHandler', el, binding)
            return {
                beforeMount() {
                    if (el.classList && !el.classList.contains('tab')) el.classList.add('tab');
                    if (!el.$vb) el.$vb = {};
                    let ins = Tab.getInstance(el);
                    if (!ins) ins = new Tab(el, binding.value);
                    el.$vb.tab = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('tab')) el.classList.add('tab');
                },
                beforeUnmount() {
                    let ins = Tab.getInstance(el);
                    if (ins) ins.dispose();
                }
            }
        },

        createToggleHandler(Tab, el, binding) {
            //console.log('createToggleHandler', el, binding)
            let clickHandler = async function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('tab toggle click', el, binding.value, el.dataset.bsTarget)
                let targetEl = null;
                if (binding.value) {
                    let refObj = binding.instance.$refs[binding.value];
                    targetEl = refObj && refObj.$el ? refObj.$el : refObj;
                } else if (el.dataset.bsTarget) {
                    targetEl = document.querySelector(el.dataset.bsTarget);
                }
                if (targetEl) {
                    let ins = Tab.getInstance(targetEl);
                    if (ins) ins.toggle(el);  // todo: toggle not defined in bs source
                }
            };
            return {
                beforeMount() {
                    //console.log('tab toggle beforemount')
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        },
    };

    var VbToast = {
        createIsHandler(Toast, el, binding) {
            //console.log('toast createIsHandler', el, binding)
            return {
                beforeMount() {
                    if (el.classList && !el.classList.contains('toast')) el.classList.add('toast');
                    if (!el.$vb) el.$vb = {};
                    let ins = Toast.getInstance(el);
                    if (!ins) ins = new Toast(el, binding.value);
                    el.$vb.modal = ins;
                },
                updated() {
                    if (el.classList && !el.classList.contains('toast')) el.classList.add('toast');
                },
                beforeUnmount() {
                    let ins = Toast.getInstance(el);
                    if (ins) ins.dispose();
                }
            }
        },

        createDismissHandler(Toast, el, binding) {
            //console.log('toast createDismissHandler', el, binding)
            let getParentToast = function(el) {
                let currNode = el;
                while (currNode) {
                    if (currNode && currNode.classList && currNode.classList.contains('toast')) {
                        break
                    }
                    currNode = currNode.parentNode;
                }
                return currNode
            };
            let clickHandler = function (e) {
                e.preventDefault();
                e.stopPropagation();
                //console.log('dismiss', binding)
                if (binding.value) {
                    let allowedToClose = binding.value(e);
                    //console.log('allowedToClose', allowedToClose)
                    if (!allowedToClose) return
                }
                let toastEl = getParentToast(e.target);
                if (toastEl) {
                    let ins = Toast.getInstance(toastEl);
                    if (ins) ins.hide();
                }
            };
            return {
                beforeMount() {
                    el.addEventListener('click', clickHandler);
                },
                beforeUnmount() {
                    el.removeEventListener('click', clickHandler);
                }
            }
        }
    };

    var VbTooltip = {

        createToggleHandler(Tooltip, el, binding) {
            //console.log('modal createToggleHandler', el)
            return {
                beforeMount() {
                    if (!el.$vb) el.$vb = {};
                    let ins = Tooltip.getInstance(el);

                    let config = undefined;
                    if (binding.value && typeof binding.value === 'string') config = {title: binding.value};
                    else if (binding.value && typeof binding.value === 'object') config = binding.value;

                    if (!ins) ins = new Tooltip(el, config);
                    el.$vb.tooltip = ins;
                },
                updated() {
                    let ins = Tooltip.getInstance(el);
                    if (ins) ins.update();
                },
                beforeUnmount() {
                    let ins = Tooltip.getInstance(el);
                    if (ins) ins.dispose();
                    el.$vb.tooltip = undefined;
                }
            }
        }
    };

    function createVbPlugin(bootstrapObjects, options) {
        return {

            /**
             * Initialize vue plugins
             *
             * @param {Object} app
             */
            install(app) {
                //console.log('createVbPlugins install', app)
                if (!bootstrapObjects) bootstrapObjects = {};
                let baseOptions = Object.assign({
                    vbModalBaseZindex: 1060  // same as $zindex-modal in https://github.com/twbs/bootstrap/blob/main/scss/_variables.scss
                }, options);

                let handlerConfs = {
                    alert: { bsObject: bootstrapObjects.Alert, handlerObject: VbAlert },
                    carousel: { bsObject: bootstrapObjects.Carousel, handlerObject: VbCarousel },
                    collapse: { bsObject: bootstrapObjects.Collapse, handlerObject: VbCollapse },
                    dropdown: { bsObject: bootstrapObjects.Dropdown, handlerObject: VbDropdown },
                    modal: { bsObject: bootstrapObjects.Modal, handlerObject: VbModal },
                    offcanvas: { bsObject: bootstrapObjects.Offcanvas, handlerObject: VbOffcanvas },
                    popover: { bsObject: bootstrapObjects.Popover, handlerObject: VbPopover },
                    scrollspy: { bsObject: bootstrapObjects.ScrollSpy, handlerObject: VbScrollSpy },
                    tab: { bsObject: bootstrapObjects.Tab, handlerObject: VbTab },
                    pill: { bsObject: bootstrapObjects.Tab, handlerObject: VbTab }, // special case for tab
                    list: { bsObject: bootstrapObjects.Tab, handlerObject: VbTab }, // special case for tab
                    toast: { bsObject: bootstrapObjects.Toast, handlerObject: VbToast },
                    tooltip: { bsObject: bootstrapObjects.Tooltip, handlerObject: VbTooltip }
                };

                let handlerObjs = [];
                let getHandler = function(directiveType, el, binding, removeHandler) {
                    //console.log('getHandler', directiveType)
                    let handlerIdx = handlerObjs.findIndex(handler => {
                        return handler.directiveType === directiveType && handler.el === el && handler.arg === binding.arg
                    });
                    //console.log('handlerIdx', handlerIdx)
                    if (handlerIdx !== -1) {
                        let handler = handlerObjs[handlerIdx];
                        if (removeHandler) handlerObjs.splice(handlerIdx, 1);
                        return handler
                    }
                    return null
                };
                let addHandler = function(directiveType, el, binding, handlerIns) {
                    handlerObjs.push({
                        directiveType: directiveType, el: el, arg: binding.arg,
                        handlerIns: handlerIns
                    });
                };

                app.directive('vb-is', {
                    created(el, binding) {
                        //console.log('is created', binding.arg, components)
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                            handlerConf.handlerObject.createIsHandler) {
                            let handlerIns = handlerConf.handlerObject.createIsHandler(handlerConf.bsObject, el, binding, baseOptions);
                            addHandler('is', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        let handler = getHandler('is', el, binding, false);
                        if (handler && handler.handlerIns.beforeMount) handler.handlerIns.beforeMount();
                    },
                    mounted(el, binding) {
                        let handler = getHandler('is', el, binding, false);
                        if (handler && handler.handlerIns.mounted) handler.handlerIns.mounted();
                    },
                    updated(el, binding) {
                        let handler = getHandler('is', el, binding, false);
                        if (handler && handler.handlerIns.updated) handler.handlerIns.updated();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('is', el, binding, false);
                        if (handler && handler.handlerIns.beforeUnmount) handler.handlerIns.beforeUnmount();
                    },
                    unmounted(el, binding) {
                        let handler = getHandler('is', el, binding, true);
                        if (handler && handler.handlerIns.unmounted) handler.handlerIns.unmounted();
                    },
                });

                app.directive('vb-toggle', {
                    created(el, binding) {
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                                handlerConf.handlerObject.createToggleHandler) {
                            let handlerIns = handlerConf.handlerObject.createToggleHandler(handlerConf.bsObject, el, binding);
                            addHandler('toggle', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        //console.log('vb-toggle beforeMount', el, binding)
                        let handler = getHandler('toggle', el, binding, false);
                        if (handler) handler.handlerIns.beforeMount();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('toggle', el, binding, true);
                        if (handler) handler.handlerIns.beforeUnmount();
                    }
                });

                app.directive('vb-dismiss', {
                    created(el, binding) {
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                            handlerConf.handlerObject.createDismissHandler) {
                            let handlerIns = handlerConf.handlerObject.createDismissHandler(handlerConf.bsObject, el, binding);
                            addHandler('dismiss', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        let handler = getHandler('dismiss', el, binding, false);
                        if (handler) handler.handlerIns.beforeMount();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('dismiss', el, binding, true);
                        if (handler) handler.handlerIns.beforeUnmount();
                    }
                });

                app.directive('vb-slide', {
                    created(el, binding) {
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                            handlerConf.handlerObject.createSlideHandler) {
                            let handlerIns = handlerConf.handlerObject.createSlideHandler(handlerConf.bsObject, el, binding);
                            addHandler('slide', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        let handler = getHandler('slide', el, binding, false);
                        if (handler) handler.handlerIns.beforeMount();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('slide', el, binding, true);
                        if (handler) handler.handlerIns.beforeUnmount();
                    }
                });

                app.directive('vb-slide-to', {
                    created(el, binding) {
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                            handlerConf.handlerObject.createSlideToHandler) {
                            let handlerIns = handlerConf.handlerObject.createSlideToHandler(handlerConf.bsObject, el, binding);
                            addHandler('slide-to', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        let handler = getHandler('slide-to', el, binding, false);
                        if (handler) handler.handlerIns.beforeMount();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('slide-to', el, binding, true);
                        if (handler) handler.handlerIns.beforeUnmount();
                    }
                });

                app.directive('vb-spy', {
                    created(el, binding) {
                        let handlerConf = handlerConfs[binding.arg];
                        if (handlerConf && handlerConf.bsObject && handlerConf.handlerObject &&
                                handlerConf.handlerObject.createSlideToHandler) {
                            let handlerIns = handlerConf.handlerObject.createSpyHandler(handlerConf.bsObject, el, binding);
                            addHandler('scrollspy', el, binding, handlerIns);
                        }
                    },
                    beforeMount(el, binding) {
                        let handler = getHandler('scrollspy', el, binding, false);
                        if (handler) handler.handlerIns.beforeMount();
                    },
                    beforeUnmount(el, binding) {
                        let handler = getHandler('scrollspy', el, binding, true);
                        if (handler) handler.handlerIns.beforeUnmount();
                    }
                });

                // not specific to a bs type

                let eventListenerPreventDefault = function(e) {
                    e.preventDefault();
                };

                app.directive('vb-set', {

                    created(el, binding) {
                        if (binding.arg) {
                            let keyName = 'vb_' + binding.arg + '_' + Object.keys(binding.modifiers).map(key => key).join('_');
                            el.dataset[keyName] = binding.value;
                        }
                    },
                    beforeMount(el, binding) {
                        if (binding.arg) {
                            let keyName = 'vb_' + binding.arg + '_' + Object.keys(binding.modifiers).map(key => key).join('_');
                            el.dataset[keyName] = binding.value;
                        }
                    },
                    updated(el, binding) {
                        if (binding.arg) {
                            let keyName = 'vb_' + binding.arg + '_' + Object.keys(binding.modifiers).map(key => key).join('_');
                            el.dataset[keyName] = binding.value;
                            if (keyName === 'vb_modal_show' && el.$vb && el.$vb.modal) {
                                if (binding.value === true && !el.$vb.modal._isShown) {
                                    el.$vb.modal.show();
                                    el.addEventListener('hide.bs.modal', eventListenerPreventDefault);
                                }
                                if (binding.value === false && el.$vb.modal._isShown) {
                                    el.removeEventListener('hide.bs.modal', eventListenerPreventDefault);
                                    el.$vb.modal.hide();
                                }
                            }
                        }
                    },
                    beforeUnmount(el, binding) {
                        if (binding.arg) {
                            let keyName = 'vb_' + binding.arg + '_' + Object.keys(binding.modifiers).map(key => key).join('_');
                            if (el.dataset[keyName]) delete el.dataset[keyName];
                        }
                    },
                });

                app.directive('vb-on', {
                    beforeMount(el, binding) {
                        //console.log('vb-on mount', el, binding.arg, binding)
                        if (binding.arg) {
                            let eventName = binding.arg + '.' + Object.keys(binding.modifiers).map(key => key).join('.');
                            console.log('eventName', eventName, binding.value, binding.expression);
                            el.addEventListener(eventName, binding.value);
                        }
                    },
                    beforeUnmount(el, binding) {
                        //console.log('vb-on beforeUnmount', el, binding.arg, binding)
                        if (binding.arg) {
                            let eventName = binding.arg + '.' + Object.keys(binding.modifiers).map(key => key).join('.');
                            el.removeEventListener(eventName, binding.value);
                        }
                    }
                });
            }
        }
    }

    exports.createVbPlugin = createVbPlugin;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
